import { PlatformContext, chainToPlatform, decimals, nativeChainAddress, nativeChainIds, networkPlatformConfigs, } from '@wormhole-foundation/connect-sdk';
import { SolanaChain } from './chain';
import { TOKEN_PROGRAM_ID } from '@solana/spl-token';
import { Connection, PublicKey, } from '@solana/web3.js';
import { SolanaAddress, SolanaZeroAddress } from './address';
import { _platform, } from './types';
/**
 * @category Solana
 */
export class SolanaPlatform extends PlatformContext {
    static _platform = _platform;
    constructor(network, config) {
        super(network, config ?? networkPlatformConfigs(network, SolanaPlatform._platform));
    }
    getRpc(chain, commitment = 'confirmed') {
        if (chain in this.config)
            return new Connection(this.config[chain].rpc, commitment);
        throw new Error('No configuration available for chain: ' + chain);
    }
    getChain(chain, rpc) {
        if (chain in this.config)
            return new SolanaChain(chain, this, rpc);
        throw new Error('No configuration available for chain: ' + chain);
    }
    static nativeTokenId(network, chain) {
        if (!SolanaPlatform.isSupportedChain(chain))
            throw new Error(`invalid chain: ${chain}`);
        return nativeChainAddress(chain, SolanaZeroAddress);
    }
    static isNativeTokenId(network, chain, tokenId) {
        if (!this.isSupportedChain(chain))
            return false;
        if (tokenId.chain !== chain)
            return false;
        const native = this.nativeTokenId(network, chain);
        return native == tokenId;
    }
    static isSupportedChain(chain) {
        const platform = chainToPlatform(chain);
        return platform === SolanaPlatform._platform;
    }
    static async getDecimals(chain, rpc, token) {
        if (token === 'native')
            return BigInt(decimals.nativeDecimals(SolanaPlatform._platform));
        let mint = await rpc.getParsedAccountInfo(new SolanaAddress(token).unwrap());
        if (!mint || !mint.value)
            throw new Error('could not fetch token details');
        const { decimals: numDecimals } = mint.value.data
            .parsed.info;
        return BigInt(numDecimals);
    }
    static async getBalance(chain, rpc, walletAddress, token) {
        if (token === 'native')
            return BigInt(await rpc.getBalance(new PublicKey(walletAddress)));
        const splToken = await rpc.getTokenAccountsByOwner(new PublicKey(walletAddress), { mint: new SolanaAddress(token).unwrap() });
        if (!splToken.value[0])
            return null;
        const balance = await rpc.getTokenAccountBalance(splToken.value[0].pubkey);
        return BigInt(balance.value.amount);
    }
    static async getBalances(chain, rpc, walletAddress, tokens) {
        let native;
        if (tokens.includes('native')) {
            native = BigInt(await rpc.getBalance(new PublicKey(walletAddress)));
        }
        const splParsedTokenAccounts = await rpc.getParsedTokenAccountsByOwner(new PublicKey(walletAddress), {
            programId: new PublicKey(TOKEN_PROGRAM_ID),
        });
        const balancesArr = tokens.map((token) => {
            if (token === 'native') {
                return { ['native']: native };
            }
            const addrString = new SolanaAddress(token).toString();
            const amount = splParsedTokenAccounts.value.find((v) => v?.account.data.parsed?.info?.mint === token)?.account.data.parsed?.info?.tokenAmount?.amount;
            if (!amount)
                return { [addrString]: null };
            return { [addrString]: BigInt(amount) };
        });
        return balancesArr.reduce((obj, item) => Object.assign(obj, item), {});
    }
    static async sendWait(chain, rpc, stxns) {
        const txhashes = await Promise.all(stxns.map((stxn) => rpc.sendRawTransaction(stxn)));
        const { blockhash, lastValidBlockHeight } = await rpc.getLatestBlockhash(rpc.commitment);
        await Promise.all(txhashes.map((txid) => {
            const bhs = {
                signature: txid,
                blockhash,
                lastValidBlockHeight,
            };
            return rpc.confirmTransaction(bhs, rpc.commitment);
        }));
        return txhashes;
    }
    static async getLatestBlock(rpc) {
        return await rpc.getSlot(rpc.commitment);
    }
    static async getLatestFinalizedBlock(rpc) {
        throw new Error('Not implemented');
        //return await rpc.getSlot(rpc.commitment);
    }
    static chainFromChainId(genesisHash) {
        const netChain = nativeChainIds.platformNativeChainIdToNetworkChain(SolanaPlatform._platform, genesisHash);
        if (!netChain)
            throw new Error(`No matching genesis hash to determine network and chain: ${genesisHash}`);
        const [network, chain] = netChain;
        return [network, chain];
    }
    static async chainFromRpc(rpc) {
        const gh = await rpc.getGenesisHash();
        return SolanaPlatform.chainFromChainId(gh);
    }
}
//# sourceMappingURL=platform.js.map